<?php

namespace Drupal\migrate_utils;

use Drupal\migrate\Event\MigrateEvents;
use Drupal\migrate\Event\MigrateImportEvent;
use Drupal\migrate\Event\MigrateRollbackEvent;
use Drupal\migrate\Plugin\MigrationInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Singleton class that holds migration state info.
 *
 * This shared service tracks the active migration using various
 * migrate events.
 * The case where a migration is not successful and does therefore not
 * trigger the POST event of import/rollback should not be a problem, because
 * a new instance of the service is created on each request / drush command.
 */
class MigrateStateService implements EventSubscriberInterface {

  /**
   * The active migration if any.
   */
  protected MigrationInterface|NULL $activeMigration = NULL;

  /**
   * If migration is running.
   */
  protected bool $migrationRunning = FALSE;

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    return [
      MigrateEvents::PRE_IMPORT => ['onPreImport'],
      MigrateEvents::POST_IMPORT => ['onPostImport'],
      MigrateEvents::PRE_ROLLBACK => ['onPreRollback'],
      MigrateEvents::POST_ROLLBACK => ['onPostRollback'],
    ];
  }

  /**
   * Handle pre import event.
   */
  public function onPreImport(MigrateImportEvent $event): void {
    $this->activeMigration = $event->getMigration();
    $this->migrationRunning = TRUE;
  }

  /**
   * Handle post import event.
   */
  public function onPostImport(MigrateImportEvent $event): void {
    $this->activeMigration = NULL;
    $this->migrationRunning = FALSE;
  }

  /**
   * Handle pre rollback event.
   */
  public function onPreRollback(MigrateRollbackEvent $event): void {
    $this->activeMigration = $event->getMigration();
    $this->migrationRunning = TRUE;
  }

  /**
   * Handle post rollback event.
   */
  public function onPostRollback(MigrateRollbackEvent $event): void {
    $this->activeMigration = NULL;
    $this->migrationRunning = FALSE;
  }

  /**
   * Whether a migration is currently running.
   */
  public function isMigrationRunning() {
    return $this->migrationRunning;
  }

}
